
# q-expansion of newform 648.2.a.f, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 648
weight = 2
poly_data = [-8, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [0, -1], [2, -1], [1, 0], [2, 1], [-3, 1], [3, 1], [2, 1], [-2, 1], [4, -1], [4, -2], [-7, 2], [3, 2], [2, -1], [-4, -2], [7, 0], [0, -1], [1, 2], [-4, 0], [-5, 3], [-4, 1], [12, 1], [6, 0], [-3, -2], [0, 3], [-10, 1], [-1, 5], [0, 2], [-2, 1], [2, -2], [16, -1], [13, -2], [1, -2], [-6, 1], [-2, -1], [4, 3], [-12, 0], [-12, 3], [-14, 1], [4, -8], [-10, -4], [-2, -5], [-3, -2], [20, 2], [-2, 6], [-12, -1], [2, 7], [-15, 0], [6, -7], [3, -1], [-2, 5], [1, -2], [17, 3], [-5, -4], [-14, 3], [-8, 2], [-12, 4], [22, -3], [-2, 5], [12, -5], [22, 1], [29, -1], [-18, -3], [-3, -6], [14, -7], [-20, 5], [-1, 8], [5, -10], [24, 3], [-5, 8], [6, -6], [20, 1], [14, -3], [-13, -3], [0, -9], [-6, -7], [-4, 6], [-3, 6], [1, -2], [4, 7], [-28, 3], [-6, -2], [25, -3], [0, -3], [7, -10], [-23, -3], [1, 2], [-14, -3], [20, -9], [5, -1], [6, 7], [2, -2], [5, 2], [-1, 8], [16, 4], [-24, 3], [5, -3], [-4, 4], [-12, -6], [-21, -2], [-14, 0], [5, -2], [13, -6], [-17, 4], [3, -5], [-5, -6], [-26, 0], [28, -5], [-13, -8], [-4, 3], [-8, 2], [19, -8], [7, -6], [-32, 0], [-11, -6], [3, 0], [10, -10], [-30, -3], [12, -3], [-20, 11], [2, 1], [-4, 7], [-13, 1], [8, -3], [-26, 4], [-12, 7], [36, -4], [18, 3], [-26, 5], [35, -3], [38, 1], [-40, -1], [18, 4], [2, 9], [26, -3], [10, -8], [12, -13], [20, 7], [25, 5], [-11, -9], [22, 1], [32, 3], [12, -8], [6, -4], [4, -11], [26, -3], [30, -3], [7, 4], [0, 8], [16, -5], [6, -8], [1, -13], [0, -5], [37, 0], [-8, -3], [22, -6], [2, -15], [30, 0], [-4, -13], [-49, 2], [-15, -3], [-6, 1], [4, 4], [5, 2], [48, -3], [32, -8], [28, -9]]
